// MapTerritory.cpp: implementation of the CMapTerritory class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "MapTerritory.h"

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CMapTerritory::CMapTerritory():CMapDataItem(MAPTYPE_TERRITORY)
{
	/****************************
	** CMapTerritory
	**
	** This is the constructor
	** for a territory object
	****************************/
	m_x = 0;	//initial location
	m_y = 0;
	m_pAdj = 0;
	m_pAdjEnd = 0;
	m_pEnd = 0;
	m_pOwner = 0;
	m_Value = 0;
	m_Color = RGB(0,0,0);
}

CMapTerritory::~CMapTerritory()
{
	/****************************
	** ~CMapTerritory
	**
	** This is the destructor
	** for a territory object
	****************************/

}

void CMapTerritory::Print(FILE *pO)
{
	/*************************
	** Print
	**
	** This function prints out
	** just this object to a
	** file
	*************************/
	fprintf(pO,"TERRITORY::\"%s\"\n",CMapDataItem::m_Name);
	fprintf(pO," LOCATION(%d,%d)\n",m_x,m_y);
	fprintf(pO," VALUE(%d)\n",m_Value);
	fprintf(pO," COLOR( RGB (%d,%d,%d))\n",(m_Color >> 16) & 0xff,(m_Color >> 8) & 0x0ff, m_Color & 0x0ff);
	//------------------------------------------------
	// check to see if we need to save an adjacent
	// neighbor list.
	//-----------------------------------------------
	if(m_pAdj)
	{
		CTerrList *pTL = m_pAdj;
		fprintf(pO,"ADJ [\n");
		while(pTL)
		{
			fprintf(pO,"\"%s\"",pTL->GetTerr()->GetName());
			pTL = pTL->GetNext();
			if(pTL)
				fprintf(pO,",\n");
			else
				fprintf(pO,"\n");
		}
		fprintf(pO,"]\n");
	}
}

void CMapTerritory::SetLocation(int x, int y)
{
	/***************************
	** SetLocation
	**
	** This function is used to
	** set the location of the
	** object.
	**
	** parameters;
	**	x.....The x location
	**	y.....The y location
	***************************/
	m_x = x;
	m_y= y;
}

void CMapTerritory::GetLocation(int *x, int *y)
{
	/**************************
	** GetLocation
	**
	** This function is used to
	** retrieve the location of
	** The object.
	**
	** parameters:
	**	x.....pointer to the location
	**       to put the X location
	**	y.....Pointer to the location
	**        to put the Y location
	**************************/
	*x = m_x;
	*y = m_y;
}

void CMapTerritory::AddNieghbor(CMapTerritory *pN)
{
	/******************************************
	** AddNiegbor
	**
	** Yes,I misspelled neightbor.  This function
	** takes a territory pointer as a parameter
	** and adds it to the Adjacent territory
	** list.
	**
	** parameter:
	**	pN....pointer to the new neighbor
	******************************************/
	if(m_pAdj)
	{
		m_pAdjEnd->SetNext(new CTerrList);
		m_pAdjEnd = m_pAdjEnd->GetNext();
		m_pAdjEnd->SetTerr(pN);
	}
	else
	{
		m_pAdj = new CTerrList;
		m_pAdj->SetTerr(pN);
		m_pAdjEnd = m_pAdj;
	}
}

void CMapTerritory::AddNieghbor(CTerrList *pN)
{
	/******************************************
	** AddNiegbor
	**
	** Yes,I misspelled neightbor.  This function
	** adds the name of an adjacent territory to
	** the adjacent territory list.  This is so
	** we can add territories before we know they
	** exist.  Right now, this is only used while
	** loading in a saved file.
	**
	** parameter:
	**	pN....pointer to the name of the new neighbor
	******************************************/
	if(m_pAdj)
	{
		m_pAdjEnd->SetNext(pN);
		m_pAdjEnd = m_pAdjEnd->GetNext();
	}
	else
	{
		m_pAdj = pN;
		m_pAdjEnd = m_pAdj;
	}
}
