// AiThread.cpp: implementation of the CAiThread class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "riskgame.h"
#include "AiThread.h"
#include "WinMsg.h"

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif


//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CAiThread::CAiThread():CMyThread(CMyThread::Worker)
{

}

CAiThread::~CAiThread()
{

}

/**********************************************
** Proc
**
** this is where we process messages from
** CRiskGameView.  Each player will have one
** of these thread descriptors, which is how
** we will keep them seperate
** currently I have the limit set at 20 players
** The data for each function call will be
** passed in the payload of an AMSG object
**********************************************/

UINT CAiThread::Proc()
{
	DWORD E;
	int count = 0;
	AMSG *pMsg;

	while(E = Wait())
	{
		if(E == MT_MESSAGE)
		{
			++count;
			//Get the message from the queue
			pMsg = m_MSGQ->Remove();
			switch(pMsg->MsgCmd)
			{
				case AIMSG_ATTACK:
					(*GetAiFuncs()->Attack)(m_pPlayer->GetParams(),m_pMap,m_pPlayer,pMsg);
					break;
				case AIMSG_ATTACKFROM:
					(*GetAiFuncs()->AttackFrom)(m_pPlayer->GetParams(),m_pMap,m_pPlayer,pMsg);
					break;
				case AIMSG_CHOOSE:
					(*GetAiFuncs()->Choose)(m_pPlayer->GetParams(),m_pMap,m_pPlayer,pMsg);
					break;
				case AIMSG_CARDS:
					break;
				case AIMSG_DEFEND:
					(*GetAiFuncs()->Defend)(m_pPlayer->GetParams(),m_pMap,m_pPlayer,pMsg);
					break;
				case AIMSG_FREEMOVE:
					(*GetAiFuncs()->FreeMove)(m_pPlayer->GetParams(),m_pMap,m_pPlayer,pMsg);
					break;
				case AIMSG_GETPARAMBLOCK:
					break;
				case AIMSG_GETPARAMETER:
					break;
				case AIMSG_GETPARAMSIZE:
					break;
				case AIMSG_GETPARAMINDEX:
					break;
				case AIMSG_GETTOTALPARAMS:
					break;
				case AIMSG_AINAME:
					break;
				case AIMSG_PLACE:
					(*GetAiFuncs()->Place)(m_pPlayer->GetParams(),m_pMap,m_pPlayer,pMsg);
					break;
				case AIMSG_SETPARAMETER:
					break;
				case AIMSG_WAR:
					(*GetAiFuncs()->War)(m_pPlayer->GetParams(),m_pMap,m_pPlayer,pMsg);
					break;
				case AIMSG_OCCUPY:
					(*GetAiFuncs()->Occupy)(m_pPlayer->GetParams(),m_pMap,m_pPlayer,pMsg);
					break;
				case AIMSG_TURNACTION:
					(*GetAiFuncs()->TurnAction)(m_pPlayer->GetParams(),m_pMap,m_pPlayer,pMsg);
					break;
				case AIMSG_RETREAT:
					(*GetAiFuncs()->Retreat)(m_pPlayer->GetParams(),m_pMap,m_pPlayer,pMsg);
					break;
			}
			//Delete the message
			DeleteAMSG(pMsg);
		}
	}
	return 0;
}

/***********************************************************
** Methodes for sending messages to the AI player
**
** Parameters:
**	Aux.......Message Dependent value for each AI function
**	payload...Pointer to a message payload.  Exactly what
**			  is sent is AI function dependent.  The
**			  message makes a copy of the data pointed
**			  to by payload, so the data pointed to by
**			  payload can be deleted, if needed without
**			  causing any problems.
**	sz........Size in bytes of the payload
***********************************************************/

void CAiThread::Choose(int Aux, void *payload,int sz)
{
	Message(AIMSG_CHOOSE,Aux,payload,sz);
}

void CAiThread::Place(int Aux,void *payload,int sz)
{
	Message(AIMSG_PLACE,Aux,payload,sz);
}

void CAiThread::Attack(int Aux,void *payload,int sz)
{
	Message(AIMSG_ATTACK,Aux,payload,sz);
}

void CAiThread::AttackFrom(int Aux,void *payload,int sz)
{
	Message(AIMSG_ATTACKFROM,Aux,payload,sz);
}


void CAiThread::War(int Aux,void *payload,int sz)
{
	Message(AIMSG_WAR,Aux,payload,sz);
}

void CAiThread::Occupy(int Aux,void *payload,int sz)
{
	Message(AIMSG_OCCUPY,Aux,payload,sz);
}

void CAiThread::TurnAction(int Aux,void *payload,int sz)
{
	Message(AIMSG_TURNACTION,Aux,payload,sz);
}

void CAiThread::FreeMove(int Aux,void *payload,int sz)
{
	Message(AIMSG_FREEMOVE,Aux,payload,sz);
}

void CAiThread::Retreat(int Aux,void *payload,int sz)
{
	Message(AIMSG_RETREAT,Aux,payload,sz);
}

void CAiThread::Cards(int Aux,void *payload,int sz)
{
	Message(AIMSG_CARDS,Aux,payload,sz);
}

void CAiThread::Message(int cmd,int Aux,void *payload,int sz)
{
	AMSG *pMSG = CreateAMSG(sz,(char *)payload,cmd);
	pMSG->hWind = m_hWnd;
	pMSG->Aux = Aux;
	pMSG->ReplyCmd = WM_MSGFROMAI;
	this->m_MSGQ->Insert(pMSG);
}